# Generated by Django 3.2.7 on 2022-03-22 19:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0006_auto_20220310_1147'),
    ]

    operations = [
        migrations.AddField(
            model_name='tablereport',
            name='order_by_ascending',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='tablereport',
            name='order_by_field',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]



from copy import deepcopy

from advanced_report_builder.models import TableReport, SingleValueReport, BarChartReport, PieChartReport, \
    FunnelChartReport, KanbanReport, LineChartReport, CustomReport
from django.shortcuts import get_object_or_404


class DuplicateReport:

    def duplicate(self, report):
        duplicate_methods = {'tablereport': self._duplicate_table_report,
                             'singlevaluereport': self._duplicate_single_value_report,
                             'barchartreport': self._duplicate_bar_chart_report,
                             'linechartreport': self._duplicate_line_chart_report,
                             'piechartreport': self._duplicate_pie_chart_report,
                             'funnelchartreport': self._duplicate_funnel_chart_report,
                             'kanbanreport': self._duplicate_kanban_report,
                             'customreport': self._duplicate_custom_report,
                             }
        new_report = duplicate_methods[report.instance_type](report_id=report.id)
        return new_report

    def _duplicate_report(self, report, copy_queries=True):
        new_report = deepcopy(report)
        new_report.id = None
        new_report.pk = None
        new_report.slug = None
        new_report.name = f'Copy {new_report.name}'
        new_report.save()
        if copy_queries:
            self._duplicate_queries(report=report, new_report=new_report)
        self._duplicate_report_tags(report=report, new_report=new_report)
        return new_report

    @staticmethod
    def _duplicate_queries(report, new_report):
        report_queries = report.reportquery_set.all()
        for report_query in report_queries:
            new_report_query = deepcopy(report_query)
            new_report_query.id = None
            new_report_query.pk = None
            new_report_query.report = new_report
            new_report_query.save()

    @staticmethod
    def _duplicate_report_tags(report, new_report):
        for report_tag in report.report_tags.all():
            new_report.report_tags.add(report_tag)

    @staticmethod
    def _duplicate_targets(report, new_report):
        for target in report.targets.all():
            new_report.targets.add(target)

    def _duplicate_table_report(self, report_id):
        table_report = get_object_or_404(TableReport, pk=report_id)
        new_table_report = self._duplicate_report(report=table_report)
        return new_table_report

    def _duplicate_single_value_report(self, report_id):
        single_value_report = get_object_or_404(SingleValueReport, pk=report_id)
        new_single_value_report = self._duplicate_report(report=single_value_report)
        return new_single_value_report

    def _duplicate_bar_chart_report(self, report_id):
        bar_chart_report = get_object_or_404(BarChartReport, pk=report_id)
        new_bar_chart_report = self._duplicate_report(report=bar_chart_report)
        return new_bar_chart_report

    def _duplicate_line_chart_report(self, report_id):
        line_chart_report = get_object_or_404(LineChartReport, pk=report_id)
        new_bar_chart_report = self._duplicate_report(report=line_chart_report)
        self._duplicate_targets(report=line_chart_report, new_report=new_bar_chart_report)
        return new_bar_chart_report

    def _duplicate_pie_chart_report(self, report_id):
        pie_report = get_object_or_404(PieChartReport, pk=report_id)
        new_pie_report = self._duplicate_report(report=pie_report)
        return new_pie_report

    def _duplicate_funnel_chart_report(self, report_id):
        funnel_chart_report = get_object_or_404(FunnelChartReport, pk=report_id)
        new_funnel_chart_report = self._duplicate_report(report=funnel_chart_report)
        return new_funnel_chart_report

    def _duplicate_kanban_report(self, report_id):
        kanban_report = get_object_or_404(KanbanReport, pk=report_id)
        new_kanban_report = self._duplicate_report(report=kanban_report, copy_queries=False)
        kanban_report_descriptions = kanban_report.kanbanreportdescription_set.all()
        descriptions_map = {}
        for kanban_report_description in kanban_report_descriptions:
            new_kanban_report_description = deepcopy(kanban_report_description)
            new_kanban_report_description.id = None
            new_kanban_report_description.pk = None
            new_kanban_report_description.kanban_report = new_kanban_report
            new_kanban_report_description.save()
            descriptions_map[kanban_report_description.id] = new_kanban_report_description

        kanban_report_lanes = kanban_report.kanbanreportlane_set.all()
        for kanban_report_lane in kanban_report_lanes:
            new_kanban_report_lane = deepcopy(kanban_report_lane)
            new_kanban_report_lane.id = None
            new_kanban_report_lane.pk = None
            new_kanban_report_lane.kanban_report = new_kanban_report
            kanban_report_description_id = kanban_report_lane.kanban_report_description_id
            new_kanban_report_lane.kanban_report_description = descriptions_map[kanban_report_description_id]
            new_kanban_report_lane.save()
        return new_kanban_report

    def _duplicate_custom_report(self, report_id):
        custom_report = get_object_or_404(CustomReport, pk=report_id)
        new_custom_report = self._duplicate_report(report=custom_report)
        return new_custom_report



# Generated by Django 3.2.7 on 2022-02-15 12:19

from django.db import migrations, models
import time_stamped_model.models


class Migration(migrations.Migration):

    dependencies = [
        ('advanced_report_builder', '0002_auto_20220209_1657'),
    ]

    operations = [
        migrations.CreateModel(
            name='Target',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', time_stamped_model.models.CreationDateTimeField(auto_now_add=True)),
                ('modified', time_stamped_model.models.ModificationDateTimeField(auto_now=True)),
                ('slug', models.SlugField(unique=True)),
                ('name', models.CharField(max_length=64)),
                ('target_type', models.PositiveSmallIntegerField(choices=[(1, 'Count'), (2, 'Money'), (3, 'Percentage')])),
                ('colour', models.CharField(blank=True, help_text=' The colour when it gets displayed on a report', max_length=10, null=True)),
                ('default_value', models.IntegerField(default=0)),
                ('overridden', models.BooleanField(default=False)),
                ('override_data', models.JSONField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='linechartreport',
            name='has_targets',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='linechartreport',
            name='targets',
            field=models.ManyToManyField(blank=True, to='advanced_report_builder.Target'),
        ),
    ]



from ajax_helpers.mixins import AjaxHelpers
from django_menus.menu import MenuItem, MenuMixin


class ReportBase(AjaxHelpers, MenuMixin):
    # noinspection PyUnusedLocal
    @staticmethod
    def duplicate_menu(request, report_id):
        return [MenuItem(f'advanced_report_builder:duplicate_report_modal,pk-{report_id}',
                         css_classes=['btn-success'])]

    def get_dashboard_class(self, report):
        return None



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.dates import MONTHS
from django_datatables.columns import DatatableColumn, NoHeadingColumn, ManyToManyColumn
from django_datatables.model_def import DatatableModel
from time_stamped_model.models import TimeStampedModel

from advanced_report_builder.globals import DISPLAY_OPTION_CHOICES, DISPLAY_OPTION_2_PER_ROW, DISPLAY_OPTION_NONE, \
    DISPLAY_OPTION_CLASSES, ANNOTATION_VALUE_CHOICES, ANNOTATIONS_CHOICES, ANNOTATION_CHOICE_COUNT, \
    ANNOTATION_CHART_SCALE


class Target(TimeStampedModel):
    TARGET_TYPE_COUNT = 1
    TARGET_TYPE_MONEY = 2
    TARGET_TYPE_PERCENTAGE = 3

    TARGET_TYPE_CHOICES = (
        (TARGET_TYPE_COUNT, 'Count'),
        (TARGET_TYPE_MONEY, 'Money'),
        (TARGET_TYPE_PERCENTAGE, 'Percentage'),
    )

    slug = models.SlugField(unique=True)
    name = models.CharField(max_length=64)
    target_type = models.PositiveSmallIntegerField(choices=TARGET_TYPE_CHOICES)
    colour = models.CharField(max_length=10, null=True, blank=True,
                              help_text='The colour when it gets displayed on a report')
    default_value = models.IntegerField(blank=True, null=True)
    default_percentage = models.FloatField(blank=True, null=True)
    overridden = models.BooleanField(default=False)
    override_data = models.JSONField(null=True, blank=True)

    def get_value(self):
        if self.target_type == self.TARGET_TYPE_PERCENTAGE:
            return self.default_percentage
        return self.default_value

    def __str__(self):
        return self.name

    def get_override_data(self):
        override_data = self.override_data if self.override_data else {}

        ordered_value_dict = {}

        for year, data in override_data.items():
            if not ordered_value_dict.get(year):
                ordered_value_dict[year] = {}
            for month in MONTHS.values():
                if data.get(str(month)):
                    ordered_value_dict[year][month] = data[month]
                else:
                    ordered_value_dict[year][month] = self.default_value

        return ordered_value_dict

    def add_year(self, year):
        override_data = self.override_data if self.override_data else {}
        if not override_data.get(str(year)):
            override_data[str(year)] = {}
        temp = {}
        # Create a temp dict that has the keys as integers.
        for key, value in override_data.items():
            temp[int(key)] = value
        sorted_dict = {}
        for key in sorted(temp):
            sorted_dict[str(key)] = temp[key]
        if sorted_dict:
            self.override_data = sorted_dict
            self.save()

    def save(self, *args, **kwargs):
        """
        Overrides the save so that a new slug is generated
        :param args:
        :param kwargs:
        """
        self.make_new_slug()
        super().save(*args, **kwargs)


class ReportTag(TimeStampedModel):
    name = models.CharField(max_length=128, unique=True)
    slug = models.SlugField(unique=True)
    order = models.PositiveSmallIntegerField()

    class Meta:
        ordering = ('order',)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.make_new_slug(allow_dashes=False, on_edit=True)
        return super().save(*args, **kwargs)


class ReportType(TimeStampedModel):
    name = models.CharField(max_length=200)
    slug = models.SlugField(unique=True)
    content_type = models.ForeignKey(ContentType, null=False, blank=False, on_delete=models.PROTECT)
    report_builder_class_name = models.CharField(max_length=200)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ['name']

    def save(self, *args, **kwargs):
        self.make_new_slug(allow_dashes=False, on_edit=True)
        return super().save(*args, **kwargs)


class Report(TimeStampedModel):
    name = models.CharField(max_length=200)
    slug = models.SlugField(unique=True)
    slug_alias = models.SlugField(blank=True, null=True)  # used if the slug changes
    report_type = models.ForeignKey(ReportType, null=True, blank=False, on_delete=models.PROTECT)
    instance_type = models.CharField(null=True, max_length=255)
    report_tags = models.ManyToManyField(ReportTag, blank=True)
    notes = models.TextField(null=True, blank=True)
    version = models.PositiveSmallIntegerField(default=0)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ['name']

    def get_report(self):
        return getattr(self, self.instance_type)

    def get_title(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.version is not None:
            self.version += 1

        slug_alias = self.slug
        self.make_new_slug(obj=Report, allow_dashes=False, on_edit=True)
        if slug_alias != self.slug:
            self.slug_alias = slug_alias

        if self.instance_type is None:
            self.instance_type = self._meta.label_lower.split('.')[1]
        return super().save(*args, **kwargs)

    def get_base_modal(self):
        return self.report_type.content_type.model_class()

    class Datatable(DatatableModel):

        class OutputType(DatatableColumn):
            output_types = {'tablereport': 'Table',
                            'singlevaluereport': 'Single Value',
                            'barchartreport': 'Bar Chart',
                            'linechartreport': 'Line Chart',
                            'piechartreport': 'Pie Chart',
                            'funnelchartreport': 'Funnel Chart',
                            'kanbanreport': 'Kanban',
                            'customreport': 'Custom'}

            def col_setup(self):
                self.field = ['instance_type', 'customreport__output_type']

            # noinspection PyMethodMayBeStatic
            def row_result(self, data, _page_data):
                instance_type = data[self.model_path + 'instance_type']
                if instance_type == 'customreport':
                    output_type = data[self.model_path + 'customreport__output_type']
                    if output_type:
                        return output_type
                return self.output_types.get(instance_type, '')

        class OutputTypeIcon(NoHeadingColumn):
            output_types = {'tablereport': '<i class="fas fa-table"></i>',
                            'singlevaluereport': '<i class="fas fa-box-open"></i>',
                            'barchartreport': '<i class="fas fa-chart-bar"></i>',
                            'linechartreport': '<i class="fas fa-chart-line"></i>',
                            'piechartreport': '<i class="fas fa-chart-pie"></i>',
                            'funnelchartreport': '<i class="fas fa-filter"></i>',
                            'kanbanreport': '<i class="fas fa-chart-bar fa-flip-vertical"></i>',
                            'customreport': '<i class="fas fa-file"></i>'}

            def col_setup(self):
                self.field = ['instance_type']

            # noinspection PyMethodMayBeStatic
            def row_result(self, data, _page_data):
                instance_type = data[self.model_path + 'instance_type']
                return self.output_types.get(instance_type, '')

        report_tags_badge = ManyToManyColumn(field='report_tags__name',
                                             html='<span class="badge badge-primary"> %1% </span>',
                                             title='Tags')


class ReportQuery(TimeStampedModel):
    report = models.ForeignKey(Report, on_delete=models.CASCADE)
    name = models.CharField(max_length=256, default='Standard')
    query = models.JSONField(null=True, blank=True)
    extra_query = models.JSONField(null=True, blank=True)  # used for single value Numerator
    order = models.PositiveSmallIntegerField()

    def save(self, *args, **kwargs):
        self.set_order_field(extra_filters={'report': self.report})
        return super().save(*args, **kwargs)

    class Meta:
        ordering = ['order']
        verbose_name_plural = 'Report queries'


class TableReport(Report):
    table_fields = models.JSONField(null=True, blank=True)
    has_clickable_rows = models.BooleanField(default=False)
    link_field = models.CharField(max_length=200, blank=True, null=True)
    pivot_fields = models.JSONField(null=True, blank=True)
    order_by_field = models.CharField(max_length=200, blank=True, null=True)
    order_by_ascending = models.BooleanField(default=True)
    page_length = models.PositiveSmallIntegerField(choices=((10, '10'),
                                                            (25, '25'),
                                                            (50, '50'),
                                                            (100, '100'),
                                                            (150, '150'),
                                                            (200, '200')), default=100)


class SingleValueReport(Report):
    SINGLE_VALUE_TYPE_COUNT = 1
    SINGLE_VALUE_TYPE_SUM = 2
    SINGLE_VALUE_TYPE_COUNT_AND_SUM = 3
    SINGLE_VALUE_TYPE_PERCENT = 4
    SINGLE_VALUE_TYPE_PERCENT_FROM_COUNT = 5
    SINGLE_VALUE_TYPE_AVERAGE = 6

    SINGLE_VALUE_TYPE_CHOICES = (
        (SINGLE_VALUE_TYPE_COUNT, 'Count'),
        (SINGLE_VALUE_TYPE_SUM, 'Sum'),
        (SINGLE_VALUE_TYPE_COUNT_AND_SUM, 'Count & Sum'),
        (SINGLE_VALUE_TYPE_PERCENT, 'Percent'),
        (SINGLE_VALUE_TYPE_PERCENT_FROM_COUNT, 'Percent from Count'),
        (SINGLE_VALUE_TYPE_AVERAGE, 'Average')
    )

    tile_colour = models.CharField(max_length=10, blank=True, null=True)
    field = models.CharField(max_length=200, blank=True, null=True)  # denominator
    numerator = models.CharField(max_length=200, blank=True, null=True)
    single_value_type = models.PositiveSmallIntegerField(choices=SINGLE_VALUE_TYPE_CHOICES,
                                                         default=SINGLE_VALUE_TYPE_COUNT)
    prefix = models.CharField(max_length=64, blank=True, null=True)
    decimal_places = models.IntegerField(default=0)
    show_breakdown = models.BooleanField(default=False)
    breakdown_fields = models.JSONField(null=True, blank=True)

    def is_percentage(self):
        return self.single_value_type in [self.SINGLE_VALUE_TYPE_PERCENT,
                                          self.SINGLE_VALUE_TYPE_PERCENT_FROM_COUNT]


class BarChartReport(Report):

    BAR_CHART_ORIENTATION_VERTICAL = 1
    BAR_CHART_ORIENTATION_HORIZONTAL = 2

    BAR_CHART_ORIENTATION_CHOICES = (
        (BAR_CHART_ORIENTATION_VERTICAL, 'Vertical'),
        (BAR_CHART_ORIENTATION_HORIZONTAL, 'Horizontal')
    )

    axis_scale = models.PositiveSmallIntegerField(choices=ANNOTATION_VALUE_CHOICES)
    date_field = models.CharField(max_length=200)
    axis_value_type = models.PositiveSmallIntegerField(choices=ANNOTATIONS_CHOICES,
                                                       default=ANNOTATION_CHOICE_COUNT, null=True, blank=True)
    fields = models.JSONField(null=True, blank=True)
    x_label = models.CharField(max_length=200, blank=True, null=True)
    y_label = models.CharField(max_length=200, blank=True, null=True)

    bar_chart_orientation = models.PositiveSmallIntegerField(choices=BAR_CHART_ORIENTATION_CHOICES,
                                                             default=BAR_CHART_ORIENTATION_VERTICAL)
    stacked = models.BooleanField(default=False)
    show_totals = models.BooleanField(default=False)
    show_blank_dates = models.BooleanField(default=True)

    def is_orientation_vertical(self):
        return self.bar_chart_orientation == self.BAR_CHART_ORIENTATION_VERTICAL

    def get_chart_scale(self):
        return ANNOTATION_CHART_SCALE[self.axis_scale]


class LineChartReport(Report):

    axis_scale = models.PositiveSmallIntegerField(choices=ANNOTATION_VALUE_CHOICES)
    date_field = models.CharField(max_length=200)
    axis_value_type = models.PositiveSmallIntegerField(choices=ANNOTATIONS_CHOICES,
                                                       default=ANNOTATION_CHOICE_COUNT, null=True, blank=True)
    fields = models.JSONField(null=True, blank=True)
    x_label = models.CharField(max_length=200, blank=True, null=True)
    y_label = models.CharField(max_length=200, blank=True, null=True)
    show_totals = models.BooleanField(default=False)

    has_targets = models.BooleanField(default=False)
    targets = models.ManyToManyField(Target, blank=True)

    def get_chart_scale(self):
        return ANNOTATION_CHART_SCALE[self.axis_scale]


class PieChartReport(Report):
    PIE_CHART_STYLE_PIE = 1
    PIE_CHART_STYLE_DOUGHNUT = 2

    PIE_CHART_STYLE_CHOICES = (
        (PIE_CHART_STYLE_PIE, 'Pie'),
        (PIE_CHART_STYLE_DOUGHNUT, 'Doughnut')
    )

    axis_value_type = models.PositiveSmallIntegerField(choices=ANNOTATIONS_CHOICES,
                                                       default=ANNOTATION_CHOICE_COUNT, null=True, blank=True)
    fields = models.JSONField(null=True, blank=True)
    style = models.PositiveSmallIntegerField(choices=PIE_CHART_STYLE_CHOICES, default=PIE_CHART_STYLE_PIE)

    def is_pie_chart(self):
        return self.style == self.PIE_CHART_STYLE_PIE


class FunnelChartReport(Report):
    axis_value_type = models.PositiveSmallIntegerField(choices=ANNOTATIONS_CHOICES,
                                                       default=ANNOTATION_CHOICE_COUNT, null=True, blank=True)
    fields = models.JSONField(null=True, blank=True)


class KanbanReport(Report):
    pass


class KanbanReportDescription(TimeStampedModel):
    kanban_report = models.ForeignKey(KanbanReport, on_delete=models.CASCADE)
    name = models.CharField(max_length=200)
    report_type = models.ForeignKey(ReportType, null=True, blank=False, on_delete=models.PROTECT)
    description = models.TextField(blank=True, null=True)
    order = models.PositiveSmallIntegerField()

    def save(self, *args, **kwargs):
        self.set_order_field(extra_filters={'kanban_report': self.kanban_report})
        return super().save(*args, **kwargs)

    def get_base_modal(self):
        return self.report_type.content_type.model_class()

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('order',)


class KanbanReportLane(TimeStampedModel):
    MULTIPLE_TYPE_NA = 0
    MULTIPLE_TYPE_DAILY = 1
    MULTIPLE_TYPE_DAILY_WITHIN = 2
    MULTIPLE_TYPE_WEEKLY = 3
    MULTIPLE_TYPE_WEEKLY_WITHIN = 4
    MULTIPLE_TYPE_MONTHLY = 5
    MULTIPLE_TYPE_MONTHLY_WITHIN = 6

    MULTIPLE_TYPE_CHOICES = [
        (MULTIPLE_TYPE_NA, 'N/A'),
        (MULTIPLE_TYPE_DAILY, 'Daily (single date)'),
        (MULTIPLE_TYPE_DAILY_WITHIN, 'Daily (within two date)'),
        (MULTIPLE_TYPE_WEEKLY, 'Weekly (single date)'),
        (MULTIPLE_TYPE_WEEKLY_WITHIN, 'Weekly (within two date)'),
        (MULTIPLE_TYPE_MONTHLY, 'Monthly (single date)'),
        (MULTIPLE_TYPE_MONTHLY_WITHIN, 'Monthly (within two date)'),
    ]

    kanban_report = models.ForeignKey(KanbanReport, on_delete=models.CASCADE)
    name = models.CharField(max_length=200)
    order = models.PositiveSmallIntegerField()
    report_type = models.ForeignKey(ReportType, null=True, blank=False, on_delete=models.PROTECT)
    heading_field = models.CharField(max_length=200, blank=True, null=True)
    link_field = models.CharField(max_length=200, blank=True, null=True)
    order_by_field = models.CharField(max_length=200, blank=True, null=True)
    order_by_ascending = models.BooleanField(default=True)
    kanban_report_description = models.ForeignKey(KanbanReportDescription,
                                                  null=True, blank=False, on_delete=models.CASCADE)

    multiple_type = models.PositiveIntegerField(choices=MULTIPLE_TYPE_CHOICES, default=MULTIPLE_TYPE_NA)
    multiple_type_label = models.CharField(max_length=200, blank=True, null=True)
    multiple_type_date_field = models.CharField(max_length=200, blank=True, null=True)
    multiple_type_end_date_field = models.CharField(max_length=200, blank=True, null=True)

    # this could be choice field from RANGE_TYPE_CHOICES however if one adds a new one it creates a new migration!
    multiple_start_period = models.PositiveSmallIntegerField(blank=True, null=True)
    multiple_end_period = models.PositiveSmallIntegerField(blank=True, null=True)
    background_colour_field = models.CharField(max_length=200, blank=True, null=True)
    heading_colour_field = models.CharField(max_length=200, blank=True, null=True)

    query_data = models.JSONField(null=True, blank=True)

    def save(self, *args, **kwargs):
        self.set_order_field(extra_filters={'kanban_report': self.kanban_report})
        return super().save(*args, **kwargs)

    def get_base_modal(self):
        return self.report_type.content_type.model_class()

    class Meta:
        ordering = ('order',)


class CustomReport(Report):
    output_type = models.CharField(max_length=200, blank=True, null=True)
    view_name = models.CharField(max_length=200)
    settings = models.JSONField(null=True, blank=True)


class Dashboard(TimeStampedModel):

    slug = models.SlugField(unique=True)
    slug_alias = models.SlugField(blank=True, null=True)  # used if the slug changes
    name = models.CharField(max_length=200)
    display_option = models.PositiveIntegerField(choices=DISPLAY_OPTION_CHOICES[1:], default=DISPLAY_OPTION_2_PER_ROW)

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        slug_alias = self.slug
        self.make_new_slug(allow_dashes=False, on_edit=True)
        if slug_alias != self.slug:
            self.slug_alias = slug_alias
        return super().save(*args, **kwargs)


class DashboardReport(TimeStampedModel):
    dashboard = models.ForeignKey(Dashboard, on_delete=models.CASCADE)
    order = models.PositiveSmallIntegerField()
    report = models.ForeignKey(Report, on_delete=models.CASCADE)
    top = models.BooleanField(default=False)
    name_override = models.CharField(max_length=200, blank=True, null=True)
    display_option = models.PositiveIntegerField(choices=DISPLAY_OPTION_CHOICES, default=DISPLAY_OPTION_NONE)

    def get_class(self, extra_class_name):
        if self.display_option != DISPLAY_OPTION_NONE:
            class_names = DISPLAY_OPTION_CLASSES.get(self.display_option)
        else:
            class_names = DISPLAY_OPTION_CLASSES.get(self.dashboard.display_option)

        if extra_class_name:
            class_names += f' {extra_class_name}'

        return class_names

    def save(self, *args, **kwargs):
        self.set_order_field(extra_filters={'dashboard': self.dashboard,
                                            'top': self.top})
        return super().save(*args, **kwargs)

    class Meta:
        ordering = ['order']



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-advanced-report-builder",
    version="0.1.5",
    author="Thomas Turner",
    description="Django app that allows you to build reports from modals",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/django-advance-utils/django-advanced-report-builder",
    include_package_data=True,
    packages=['advanced_report_builder'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'Django >= 3.2',
        'django-filtered-datatables >= 0.0.21',
        'django-ajax-helpers >= 0.0.20',
        'django-nested-modals >= 0.0.21',
        'time-stamped-model >= 0.2.3',
        'date-offset >= 0.0.2'
    ],

)



from django.urls import path

from advanced_report_builder.views.bar_charts import BarChartModal, BarChartFieldModal
from advanced_report_builder.views.custom import CustomModal
from advanced_report_builder.views.dashboard import DashboardReportModal, DashboardModal, DashboardAddReportModal
from advanced_report_builder.views.datatables.modal import TableModal, TableFieldModal, TablePivotModal, TableQueryModal
from advanced_report_builder.views.funnel_charts import FunnelChartModal, FunnelChartFieldModal
from advanced_report_builder.views.kanban import KanbanModal, KanbanLaneModal, KanbanLaneDuplicateModal, \
    KanbanDescriptionModal, KanbanDescriptionDuplicateModal
from advanced_report_builder.views.line_charts import LineChartModal, LineChartFieldModal
from advanced_report_builder.views.pie_charts import PieChartModal, PieChartFieldModal
from advanced_report_builder.views.reports import DuplicateReportModal
from advanced_report_builder.views.single_values import SingleValueModal, ShowBreakdownModal, SingleValueTableFieldModal
from advanced_report_builder.views.targets.views import TargetModal

app_name = 'advanced_report_builder'


urlpatterns = [
    path('table/modal/<str:slug>/', TableModal.as_view(), name='table_modal'),
    path('table/modal/field/<str:slug>/', TableFieldModal.as_view(), name='table_field_modal'),
    path('table/modal/pivot/<str:slug>/', TablePivotModal.as_view(), name='table_pivot_modal'),
    path('table/modal/query/<str:slug>', TableQueryModal.as_view(), name='table_query_modal'),

    path('single-value/modal/<str:slug>/', SingleValueModal.as_view(), name='single_value_modal'),
    path('single-value/modal/field/<str:slug>/', SingleValueTableFieldModal.as_view(), name='single_value_field_modal'),
    path('single-value/show-breakdown/<str:slug>/', ShowBreakdownModal.as_view(), name='show_breakdown_modal'),

    path('bar-chart/modal/<str:slug>/', BarChartModal.as_view(), name='bar_chart_modal'),
    path('bar-chart/modal/field/<str:slug>/', BarChartFieldModal.as_view(), name='bar_chart_field_modal'),

    path('line-chart/modal/<str:slug>/', LineChartModal.as_view(), name='line_chart_modal'),
    path('line-chart/modal/field/<str:slug>/', LineChartFieldModal.as_view(), name='line_chart_field_modal'),

    path('pie-chart/modal/<str:slug>/', PieChartModal.as_view(), name='pie_chart_modal'),
    path('pie-chart/modal/field/<str:slug>/', PieChartFieldModal.as_view(), name='pie_chart_field_modal'),

    path('funnel-chart/modal/<str:slug>/', FunnelChartModal.as_view(), name='funnel_chart_modal'),
    path('funnel-chart/modal/field/<str:slug>/', FunnelChartFieldModal.as_view(), name='funnel_chart_field_modal'),

    path('kanban/modal/<str:slug>/', KanbanModal.as_view(), name='kanban_modal'),
    path('kanban/modal/lane/<str:slug>/', KanbanLaneModal.as_view(), name='kanban_lane_modal'),
    path('kanban/modal/lane/duplicate/<str:slug>/', KanbanLaneDuplicateModal.as_view(),
         name='kanban_lane_duplicate_modal'),
    path('kanban/modal/description/<str:slug>/', KanbanDescriptionModal.as_view(), name='kanban_description_modal'),
    path('kanban/modal/description/duplicate/<str:slug>/', KanbanDescriptionDuplicateModal.as_view(),
         name='kanban_description_duplicate_modal'),

    path('custom/modal/<str:slug>/', CustomModal.as_view(), name='custom_modal'),

    path('dashboard/report/<str:slug>/', DashboardReportModal.as_view(), name='dashboard_report_modal'),
    path('dashboard/<str:slug>/', DashboardModal.as_view(), name='dashboard_modal'),
    path('dashboard/add/<str:slug>/', DashboardAddReportModal.as_view(), name='add_dashboard_report'),

    path('duplicate/<str:slug>/', DuplicateReportModal.as_view(), name='duplicate_report_modal'),

    path('target/modal/<str:slug>/', TargetModal.as_view(), name='target_modal'),
]
